@@ -53,33 +53,46 @@ def compare_obj(test, obj, db_obj, subs=None, allow_missing=None,
     :param comparators: Map of comparator functions to use for certain fields
     """
 
-    if subs is None:
-        subs = {}
-    if allow_missing is None:
-        allow_missing = []
-    if comparators is None:
-        comparators = {}
+    subs = subs or {}
+    allow_missing = allow_missing or []
+    comparators = comparators or {}
 
     for key in obj.fields:
-        # We'll raise a NotImplementedError if we try to compare against
-        # against something that isn't set in the object, but is not
-        # in the allow_missing. This will replace that exception
-        # with an AssertionError (because that is a better way of saying
-        # "these objects arent the same").
-        if not obj.obj_attr_is_set(key):
-            if key in allow_missing:
-                continue
-            else:
-                raise AssertionError(("%s is not set on the object, so "
-                                      "the objects are not equal") % key)
-        if key in allow_missing and not obj.obj_attr_is_set(key):
-            continue
-        obj_val = getattr(obj, key)
         db_key = subs.get(key, key)
+
+        # If this is an allow_missing key and it's missing in either obj or
+        # db_obj, just skip it
+        if key in allow_missing:
+            if key not in obj or db_key not in db_obj:
+                continue
+
+        # If the value isn't set on the object, and also isn't set on the
+        # db_obj, we'll skip the value check, unset in both is equal
+        if not obj.obj_attr_is_set(key) and db_key not in db_obj:
+            continue
+        # If it's set on the object and not on the db_obj, they aren't equal
+        elif obj.obj_attr_is_set(key) and db_key not in db_obj:
+            raise AssertionError(("%s (db_key: %s) is set on the object, but "
+                                  "not on the db_obj, so the objects are not "
+                                  "equal")
+                                 % (key, db_key))
+        # If it's set on the db_obj and not the object, they aren't equal
+        elif not obj.obj_attr_is_set(key) and db_key in db_obj:
+            raise AssertionError(("%s (db_key: %s) is set on the db_obj, but "
+                                  "not on the object, so the objects are not "
+                                  "equal")
+                                 % (key, db_key))
+
+        # All of the checks above have safeguarded us, so we know we will
+        # get an obj_val and db_val without issue
+        obj_val = getattr(obj, key)
         db_val = db_obj[db_key]
         if isinstance(obj_val, datetime.datetime):
             obj_val = obj_val.replace(tzinfo=None)
 
+        if isinstance(db_val, datetime.datetime):
+            db_val = obj_val.replace(tzinfo=None)
+
         if key in comparators:
             comparator = comparators[key]
             comparator(db_val, obj_val)
@@ -252,8 +265,8 @@ class ObjectVersionChecker(object):
                 obj_name, extra_data_func=extra_data_func)
         return fingerprints
 
-    def test_hashes(self, expected_hashes):
-        fingerprints = self.get_hashes()
+    def test_hashes(self, expected_hashes, extra_data_func=None):
+        fingerprints = self.get_hashes(extra_data_func=extra_data_func)
 
         stored = set(expected_hashes.items())
         computed = set(fingerprints.items())
@@ -399,3 +412,33 @@ class VersionedObjectRegistryFixture(fixtures.Fixture):
     def _restore_obj_registry(self):
         base.VersionedObjectRegistry._registry._obj_classes = \
             self._base_test_obj_backup
+
+
+class StableObjectJsonFixture(fixtures.Fixture):
+    """Fixture that makes sure we get stable JSON object representations.
+
+    Since objects contain things like set(), which can't be converted to
+    JSON, we have some situations where the representation isn't fully
+    deterministic. This doesn't matter at all at runtime, but does to
+    unit tests that try to assert things at a low level.
+
+    This fixture mocks the obj_to_primitive() call and makes sure to
+    sort the list of changed fields (which came from a set) before
+    returning it to the caller.
+    """
+    def __init__(self):
+        self._original_otp = base.VersionedObject.obj_to_primitive
+
+    def setUp(self):
+        super(StableObjectJsonFixture, self).setUp()
+
+        def _doit(obj, *args, **kwargs):
+            result = self._original_otp(obj, *args, **kwargs)
+            changes_key = obj._obj_primitive_key('changes')
+            if changes_key in result:
+                result[changes_key].sort()
+            return result
+
+        self.useFixture(fixtures.MonkeyPatch(
+            'oslo_versionedobjects.base.VersionedObject.obj_to_primitive',
+            _doit))
