@@ -37,6 +37,9 @@ class FakeFieldType(fields.FieldType):
     def from_primitive(self, obj, attr, value):
         return value[1:-1]
 
+    def get_schema(self):
+        return {'type': ['foo']}
+
 
 class FakeEnum(fields.Enum):
     FROG = "frog"
@@ -55,7 +58,7 @@ class FakeEnumAlt(fields.Enum):
     PLATYPUS = "platypus"
     AARDVARK = "aardvark"
 
-    ALL = (FROG, PLATYPUS, AARDVARK)
+    ALL = set([FROG, PLATYPUS, AARDVARK])
 
     def __init__(self, **kwargs):
         super(FakeEnumAlt, self).__init__(valid_values=FakeEnumAlt.ALL,
@@ -96,6 +99,11 @@ class FakeEnumAltField(fields.BaseEnumField):
     AUTO_TYPE = FakeEnumAlt()
 
 
+class TestFieldType(test.TestCase):
+    def test_get_schema(self):
+        self.assertRaises(NotImplementedError, fields.FieldType().get_schema)
+
+
 class TestField(test.TestCase):
     def setUp(self):
         super(TestField, self).setUp()
@@ -131,6 +139,18 @@ class TestField(test.TestCase):
         self.assertEqual('123', self.field.stringify(123))
 
 
+class TestSchema(test.TestCase):
+    def setUp(self):
+        super(TestSchema, self).setUp()
+        self.field = fields.Field(FakeFieldType(), nullable=True,
+                                  default='', read_only=False)
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['foo', 'null'], 'default': '',
+                          'readonly': False},
+                         self.field.get_schema())
+
+
 class TestString(TestField):
     def setUp(self):
         super(TestString, self).setUp()
@@ -146,6 +166,13 @@ class TestString(TestField):
     def test_stringify(self):
         self.assertEqual("'123'", self.field.stringify(123))
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'type': ['string']}, self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'readonly': False},
+                         self.field.get_schema())
+
 
 class TestSensitiveString(TestString):
     def setUp(self):
@@ -190,12 +217,53 @@ class TestMACAddress(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
+
+class TestPCIAddress(TestField):
+    def setUp(self):
+        super(TestPCIAddress, self).setUp()
+        self.field = fields.PCIAddressField()
+        self.coerce_good_values = [
+            ('0000:02:00.0', '0000:02:00.0'),
+            ('FFFF:FF:1F.7', 'ffff:ff:1f.7'),
+            ('fFfF:fF:1F.7', 'ffff:ff:1f.7'),
+        ]
+        self.coerce_bad_values = [
+            '000:02:00.0',  # Too short
+            '00000:02:00.0',  # Too long
+            'FFFF:FF:2F.7',  # Bad slot
+            'FFFF:GF:1F.7',  # Bad octal
+            1123123,  # Number
+            {},  # dict
+        ]
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
 
 class TestUUID(TestField):
     def setUp(self):
         super(TestUUID, self).setUp()
-
-    def test_validation_enabled(self):
         self.field = fields.UUIDField()
         self.coerce_good_values = [
             ('da66a411-af0e-4829-9b67-475017ddd152',
@@ -219,14 +287,28 @@ class TestUUID(TestField):
             ('1', '1'),
             (1, '1')
         ]
-
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_validation_enabled(self):
+
         self.test_coerce_good_values()
         self.test_from_primitive()
         self.test_to_primitive()
 
+    def test_get_schema(self):
+        field = fields.UUIDField()
+        schema = field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        print(self.coerce_good_values)
+        for _, valid_val in self.coerce_good_values[:4]:
+            self.assertRegex(valid_val, pattern)
+        invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(invalid_val, pattern)
+
 
 class TestBaseEnum(TestField):
     def setUp(self):
@@ -252,6 +334,14 @@ class TestBaseEnum(TestField):
         field2 = FakeEnumAltField()
         self.assertNotEqual(str(field1), str(field2))
 
+    def test_valid_values(self):
+        self.assertEqual(self.field.valid_values,
+                         FakeEnum.ALL)
+
+    def test_valid_values_keeps_type(self):
+        self.assertIsInstance(self.field.valid_values, tuple)
+        self.assertIsInstance(FakeEnumAltField().valid_values, set)
+
 
 class TestEnum(TestField):
     def setUp(self):
@@ -269,6 +359,14 @@ class TestEnum(TestField):
     def test_stringify_invalid(self):
         self.assertRaises(ValueError, self.field.stringify, '123')
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'enum': ["foo", "bar", 1, True]},
+                         self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'enum': ["foo", "bar", 1, True],
+                          'readonly': False}, self.field.get_schema())
+
     def test_fingerprint(self):
         # Notes(yjiang5): make sure changing valid_value will be detected
         # in test_objects.test_versions
@@ -288,6 +386,15 @@ class TestEnum(TestField):
         self.assertRaises(exception.EnumValidValuesInvalidError,
                           fields.EnumField, True)
 
+    def test_enum_subclass_check(self):
+        def _test():
+            class BrokenEnumField(fields.BaseEnumField):
+                AUTO_TYPE = int
+
+            BrokenEnumField()
+
+        self.assertRaises(exception.EnumFieldInvalid, _test)
+
 
 class TestStateMachine(TestField):
 
@@ -363,6 +470,23 @@ class TestInteger(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'type': ['integer']}, self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['integer'], 'readonly': False},
+                         self.field.get_schema())
+
+
+class TestNonNegativeInteger(TestField):
+    def setUp(self):
+        super(TestNonNegativeInteger, self).setUp()
+        self.field = fields.NonNegativeIntegerField()
+        self.coerce_good_values = [(1, 1), ('1', 1)]
+        self.coerce_bad_values = ['-2', '4.2', 'foo', None]
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
 
 class TestFloat(TestField):
     def setUp(self):
@@ -373,6 +497,23 @@ class TestFloat(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'type': ['number']}, self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['number'], 'readonly': False},
+                         self.field.get_schema())
+
+
+class TestNonNegativeFloat(TestField):
+    def setUp(self):
+        super(TestNonNegativeFloat, self).setUp()
+        self.field = fields.NonNegativeFloatField()
+        self.coerce_good_values = [(1.1, 1.1), ('1.1', 1.1)]
+        self.coerce_bad_values = ['-4.2', 'foo', None]
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
 
 class TestBoolean(TestField):
     def setUp(self):
@@ -384,6 +525,13 @@ class TestBoolean(TestField):
         self.to_primitive_values = self.coerce_good_values[0:2]
         self.from_primitive_values = self.coerce_good_values[0:2]
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'type': ['boolean']}, self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['boolean'], 'readonly': False},
+                         self.field.get_schema())
+
 
 class TestFlexibleBoolean(TestField):
     def setUp(self):
@@ -420,6 +568,11 @@ class TestDateTime(TestField):
                 datetime.datetime(1955, 11, 5, 18, 0, 0,
                                   tzinfo=iso8601.iso8601.Utc())))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'format': 'date-time',
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestDateTimeNoTzinfo(TestField):
     def setUp(self):
@@ -457,6 +610,10 @@ class TestDict(TestField):
     def test_stringify(self):
         self.assertEqual("{key=val}", self.field.stringify({'key': 'val'}))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['object'], 'readonly': False},
+                         self.field.get_schema())
+
 
 class TestDictOfStrings(TestField):
     def setUp(self):
@@ -539,6 +696,17 @@ class TestList(TestField):
     def test_stringify(self):
         self.assertEqual('[123]', self.field.stringify([123]))
 
+    def test_fieldtype_get_schema(self):
+        self.assertEqual({'type': ['array'],
+                          'items': {'type': ['foo'], 'readonly': False}},
+                         self.field._type.get_schema())
+
+    def test_get_schema(self):
+        self.assertEqual({'type': ['array'],
+                          'items': {'type': ['foo'], 'readonly': False},
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestListOfStrings(TestField):
     def setUp(self):
@@ -605,6 +773,12 @@ class TestSet(TestField):
     def test_stringify(self):
         self.assertEqual('set([123])', self.field.stringify(set([123])))
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['array'], 'uniqueItems': True,
+                          'items': {'type': ['foo'], 'readonly': False},
+                          'readonly': False},
+                         self.field.get_schema())
+
 
 class TestSetOfIntegers(TestField):
     def setUp(self):
@@ -634,6 +808,20 @@ class TestListOfSetsOfIntegers(TestField):
         self.assertEqual('[set([1,2])]', self.field.stringify([set([1, 2])]))
 
 
+class TestListOfIntegers(TestField):
+    def setUp(self):
+        super(TestListOfIntegers, self).setUp()
+        self.field = fields.ListOfIntegersField()
+        self.coerce_good_values = [(['1', 2], [1, 2]),
+                                   ([1, 2], [1, 2])]
+        self.coerce_bad_values = [['foo']]
+        self.to_primitive_values = [([1], [1])]
+        self.from_primitive_values = [([1], [1])]
+
+    def test_stringify(self):
+        self.assertEqual('[[1, 2]]', self.field.stringify([[1, 2]]))
+
+
 class TestLocalMethods(test.TestCase):
     @mock.patch.object(obj_base.LOG, 'exception')
     def test__make_class_properties_setter_value_error(self, mock_log):
@@ -815,6 +1003,13 @@ class TestObject(TestField):
         self.assertRaises(ValueError, pigs.coerce, None, "animal", wolfy)
         self.assertEqual(babe, pigs.coerce(None, "animal", babe))
 
+    def test_coerce_bad_value_primitive_type(self):
+        # Tests that the ValueError has the primitive type in it's message.
+        ex = self.assertRaises(ValueError, self.field.coerce,
+                               'obj', 'attr', [{}])
+        self.assertEqual('An object of type TestableObject is required '
+                         'in field attr, not a list', six.text_type(ex))
+
 
 class TestIPAddress(TestField):
     def setUp(self):
@@ -845,6 +1040,11 @@ class TestIPAddressV4(TestField):
         self.from_primitive_values = [('1.2.3.4',
                                        netaddr.IPAddress('1.2.3.4'))]
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'readonly': False,
+                          'format': 'ipv4'},
+                         self.field.get_schema())
+
 
 class TestIPAddressV6(TestField):
     def setUp(self):
@@ -858,6 +1058,33 @@ class TestIPAddressV6(TestField):
         self.from_primitive_values = [('::1',
                                        netaddr.IPAddress('::1'))]
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['string'], 'readonly': False,
+                          'format': 'ipv6'},
+                         self.field.get_schema())
+
+
+class TestIPV4AndV6Address(TestField):
+    def setUp(self):
+        super(TestIPV4AndV6Address, self).setUp()
+        self.field = fields.IPV4AndV6Address()
+        self.coerce_good_values = [('::1', netaddr.IPAddress('::1')),
+                                   (netaddr.IPAddress('::1'),
+                                    netaddr.IPAddress('::1')),
+                                   ('1.2.3.4',
+                                    netaddr.IPAddress('1.2.3.4')),
+                                   (netaddr.IPAddress('1.2.3.4'),
+                                    netaddr.IPAddress('1.2.3.4'))]
+        self.coerce_bad_values = ['1-2', 'foo']
+        self.to_primitive_values = [(netaddr.IPAddress('::1'),
+                                     '::1'),
+                                    (netaddr.IPAddress('1.2.3.4'),
+                                     '1.2.3.4')]
+        self.from_primitive_values = [('::1',
+                                       netaddr.IPAddress('::1')),
+                                      ('1.2.3.4',
+                                       netaddr.IPAddress('1.2.3.4'))]
+
 
 class TestIPNetwork(TestField):
     def setUp(self):
@@ -886,6 +1113,17 @@ class TestIPV4Network(TestField):
         self.from_primitive_values = [('1.2.3.4/24',
                                        netaddr.IPNetwork('1.2.3.4/24'))]
 
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(str(valid_val), pattern)
+        invalid_vals = [x for x in self.coerce_bad_values]
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(str(invalid_val), pattern)
+
 
 class TestIPV6Network(TestField):
     def setUp(self):
@@ -898,3 +1136,14 @@ class TestIPV6Network(TestField):
         self.to_primitive_values = [(netaddr.IPNetwork('::1/0'), '::1/0')]
         self.from_primitive_values = [('::1/0',
                                        netaddr.IPNetwork('::1/0'))]
+
+    def test_get_schema(self):
+        schema = self.field.get_schema()
+        self.assertEqual(['string'], schema['type'])
+        self.assertEqual(False, schema['readonly'])
+        pattern = schema['pattern']
+        for _, valid_val in self.coerce_good_values:
+            self.assertRegex(str(valid_val), pattern)
+        invalid_vals = [x for x in self.coerce_bad_values]
+        for invalid_val in invalid_vals:
+            self.assertNotRegex(str(invalid_val), pattern)
